@@ -120,6 +120,9 @@ class NumericField(Field):
         if stop is not None:
             stop = None if stop > lucene.Long.MAX_VALUE else lucene.Long(long(stop))
         return lucene.NumericRangeQuery.newLongRange(self.name, self.step, start, stop, lower, upper)
+    def term(self, value):
+        "Return range query to match single term."
+        return self.range(value, value, upper=True)
 
 class DateTimeField(NumericField):
     """Field which indexes datetimes as a NumericField of timestamps.
@@ -246,3 +249,16 @@ class Hits(object):
     def items(self):
         "Generate zipped ids and scores."
         return zip(self.ids, self.scores)
+    def groupby(self, func):
+        "Return ordered list of `Hits`_ grouped by value of function applied to doc ids."
+        groups = {}
+        for id, score in self.items():
+            value = func(id)
+            try:
+                group = groups[value]
+            except KeyError:
+                group = groups[value] = type(self)(self.searcher, [], [], fields=self.fields)
+                group.index, group.value = len(groups), value
+            group.ids.append(id)
+            group.scores.append(score)
+        return sorted(groups.values(), key=lambda group: group.__dict__.pop('index'))
