@@ -5,7 +5,6 @@ The final `Indexer`_ classes exposes a high-level Searcher and Writer.
 """
 
 from future_builtins import map, zip
-import warnings
 import os
 import re
 import itertools, operator
@@ -14,6 +13,7 @@ import abc, collections
 import lucene
 from .queries import Query, Collector, SortField, Highlighter, FastVectorHighlighter, SpellChecker, SpellParser
 from .documents import Field, Document, Hits
+from .spatial import DistanceComparator
 
 class Atomic(object):
     "Abstract base class to distinguish singleton values from other iterables."
@@ -44,15 +44,31 @@ class closing(set):
         return directory
     def reader(self, reader):
         reader = self.directory(reader)
-        if isinstance(reader, lucene.Searcher):
-            warnings.warn('Support for opening searchers is deprecated; call with a reader instead.', DeprecationWarning)
-            reader = reader.indexReader
         if isinstance(reader, lucene.IndexReader):
             reader.incRef()
         else:
-            reader = lucene.IndexReader.open(reader)
+            reader = lucene.IndexReader.open(reader, True)
         return reader
 
+def copy(commit, dest):
+    """Copy the index commit to the destination directory.
+    Optimized to use hard links if the destination is a file system path.
+    """
+    src = commit.directory
+    if isinstance(dest, lucene.Directory):
+        for filename in commit.fileNames:
+            src.copy(dest, filename, filename)
+    else:
+        src = lucene.FSDirectory.cast_(src).file.path
+        os.path.isdir(dest) or os.makedirs(dest)
+        for filename in commit.fileNames:
+            paths = os.path.join(src, filename), os.path.join(dest, filename)
+            try:
+                os.link(*paths)
+            except OSError:
+                if not os.path.samefile(*paths):
+                    raise
+
 class TokenFilter(lucene.PythonTokenFilter):
     """Create an iterable lucene TokenFilter from a TokenStream.
     Subclass and override :meth:`incrementToken`.
@@ -116,7 +132,7 @@ class Analyzer(lucene.PythonAnalyzer):
     """Return a lucene Analyzer which chains together a tokenizer and filters.
     
     :param tokenizer: lucene Tokenizer or Analyzer
-    :param filters: lucene TokenFilters or python generators.
+    :param filters: lucene TokenFilters
     """
     def __init__(self, tokenizer, *filters):
         lucene.PythonAnalyzer.__init__(self)
@@ -201,20 +217,13 @@ class IndexReader(object):
         :param exclude: optional lucene Query to exclude documents
         :param optimize: optionally optimize destination index
         """
-        filter = lucene.IndexFileNameFilter.getFilter()
-        filenames = (filename for filename in self.directory.listAll() if filter.accept(None, filename))
         if isinstance(dest, lucene.Directory):
-            src = self.directory
             try:
-                lucene.Directory.copy(src, dest, False)
+                lucene.Directory.copy(self.directory, dest, False)
             except lucene.InvalidArgsError:
-                for filename in filenames:
-                    src.copy(dest, filename, filename)
+                copy(self.indexCommit, dest)
         else:
-            src = lucene.FSDirectory.cast_(self.directory).file.path
-            os.path.isdir(dest) or os.makedirs(dest)
-            for filename in filenames:
-                os.link(os.path.join(src, filename), os.path.join(dest, filename))
+            copy(self.indexCommit, dest)
         with contextlib.closing(IndexWriter(dest)) as writer:
             if query:
                 writer.delete(Query(lucene.MatchAllDocsQuery) - query)
@@ -342,14 +351,12 @@ class IndexReader(object):
         return mlt.like(lucene.StringReader(doc) if isinstance(doc, basestring) else doc)
     def overlap(self, left, right):
         "Return intersection count of cached filters."
-        count = 0
+        count, bitset = 0, getattr(lucene, 'FixedBitSet', lucene.OpenBitSet)
         for reader in self.sequentialSubReaders:
             docsets = left.getDocIdSet(reader), right.getDocIdSet(reader)
-            try:
-                count += lucene.OpenBitSet.intersectionCount(*docsets)
-            except lucene.InvalidArgsError: # verify docsets are either cached or empty
-                if not all(lucene.OpenBitSet.instance_(docset) or docset.iterator().nextDoc() == lucene.Integer.MAX_VALUE for docset in docsets):
-                    raise
+            if lucene.DocIdSet.EMPTY_DOCIDSET not in docsets:
+                bits = [bitset.cast_(docset).bits for docset in docsets]
+                count += lucene.BitUtil.pop_intersect(bits[0], bits[1], 0, min(map(len, bits)))
         return int(count)
 
 class IndexSearcher(lucene.IndexSearcher, IndexReader):
@@ -387,7 +394,7 @@ class IndexSearcher(lucene.IndexSearcher, IndexReader):
         other.filters.update((key, value if isinstance(value, lucene.Filter) else dict(value)) for key, value in self.filters.items())
         if filters:
             other.facets(Query.any(), *other.filters)
-        other.sorters = dict(self.sorters)
+        other.sorters = dict((name, SortField(sorter.field, sorter.typename, sorter.parser)) for name, sorter in self.sorters.items())
         if sorters:
             for field in self.sorters:
                 other.comparator(field)
@@ -441,7 +448,7 @@ class IndexSearcher(lucene.IndexSearcher, IndexReader):
         :param parser: :meth:`Analyzer.parse` options
         """
         query = lucene.MatchAllDocsQuery() if query is None else self.parse(query, **parser)
-        weight = query.weight(self)
+        weight = self.createNormalizedWeight(query) if hasattr(self, 'createNormalizedWeight') else query.weight(self)
         # use custom Collector if all results are necessary, otherwise use lucene's TopDocsCollectors
         if count is None:
             collector = Collector()
@@ -475,7 +482,7 @@ class IndexSearcher(lucene.IndexSearcher, IndexReader):
     def facets(self, query, *keys):
         """Return mapping of document counts for the intersection with each facet.
         
-        :param ids: query string, lucene Query, or lucene Filter
+        :param query: query string, lucene Query, or lucene Filter
         :param keys: field names, term tuples, or any keys to previously cached filters
         """
         counts = collections.defaultdict(dict)
@@ -507,7 +514,14 @@ class IndexSearcher(lucene.IndexSearcher, IndexReader):
         return sorter if sorter.reverse == reverse else SortField(sorter.field, sorter.typename, sorter.parser, reverse)
     def comparator(self, field, type='string', parser=None):
         "Return :meth:`IndexReader.comparator` using a cached `SortField`_ if available."
-        return self.sorter(field, type, parser).comparator(self.indexReader)
+        sorter = self.sorter(field, type, parser)
+        if not hasattr(sorter, 'cache'):
+            sorter.cache = sorter.comparator(self.indexReader)
+        return sorter.cache
+    def distances(self, lng, lat, lngfield, latfield):
+        "Return distance comparator computed from cached lat/lng fields."
+        arrays = (self.comparator(field, 'double') for field in (lngfield, latfield))
+        return DistanceComparator(lng, lat, *arrays)
     def spellchecker(self, field):
         "Return and cache spellchecker for given field."
         try:
@@ -569,11 +583,6 @@ class MultiSearcher(IndexSearcher):
     def overlap(self, *filters):
         return sum(IndexReader(reader).overlap(*filters) for reader in self.sequentialSubReaders)
 
-class ParallelMultiSearcher(MultiSearcher):
-    def __init__(self, *args, **kwargs):
-        warnings.warn('ParallelMultiSearchers have been deprecated; use MultiSearcher.', DeprecationWarning)
-        MultiSearcher.__init__(self, *args, **kwargs)
-
 class IndexWriter(lucene.IndexWriter):
     """Inherited lucene IndexWriter.
     Supports setting fields parameters explicitly, so documents can be represented as dictionaries.
@@ -592,19 +601,17 @@ class IndexWriter(lucene.IndexWriter):
             version = lucene.Version.values()[-1]
         analyzer = self.shared.analyzer(analyzer, version)
         directory = self.shared.directory(directory)
+        self.policy = lucene.SnapshotDeletionPolicy(lucene.KeepOnlyLastCommitDeletionPolicy())
         if hasattr(lucene, 'IndexWriterConfig'):
             config = lucene.IndexWriterConfig(version, analyzer)
             config.openMode = lucene.IndexWriterConfig.OpenMode.values()['wra'.index(mode)]
+            config.indexDeletionPolicy = self.policy
             lucene.IndexWriter.__init__(self, directory, config)
         else:
             args = [] if mode == 'a' else [bool('rw'.index(mode))]
-            args.append(lucene.IndexWriter.MaxFieldLength.UNLIMITED)
+            args += self.policy, lucene.IndexWriter.MaxFieldLength.UNLIMITED
             lucene.IndexWriter.__init__(self, directory, analyzer, *args)
         self.fields = {}
-    @property
-    def segments(self):
-        warnings.warn('IndexWriter.segString is for Lucene internal purposes only', DeprecationWarning)
-        return dict((name, int(value)) for name, value in re.findall('(\w+).*?:.x?(\d+)', self.segString()))
     def set(self, name, cls=Field, **params):
         """Assign parameters to field name.
         
@@ -613,24 +620,22 @@ class IndexWriter(lucene.IndexWriter):
         :param params: store,index,termvector options compatible with `Field`_
         """
         self.fields[name] = cls(name, **params)
-    def document(self, document=(), **terms):
+    def document(self, document=(), boost=1.0, **terms):
         "Return lucene Document from mapping of field names to one or multiple values."
         doc = lucene.Document()
+        doc.boost = boost
         for name, values in dict(document, **terms).items():
             if isinstance(values, Atomic):
                 values = values,
             for field in self.fields[name].items(*values):
                 doc.add(field)
         return doc
-    def add(self, document=(), boost=1.0, **terms):
+    def add(self, document=(), **terms):
         "Add :meth:`document` to index with optional boost."
-        doc = self.document(document, **terms)
-        doc.boost = boost
-        self.addDocument(doc)
-    def update(self, name, value='', document=(), boost=1.0, **terms):
+        self.addDocument(self.document(document, **terms))
+    def update(self, name, value='', document=(), **terms):
         "Atomically delete documents which match given term and add the new :meth:`document` with optional boost."
         doc = self.document(document, **terms)
-        doc.boost = boost
         self.updateDocument(lucene.Term(name, *[value] if value else doc.getValues(name)), doc)
     def delete(self, *query, **options):
         """Remove documents which match given query or term.
@@ -642,12 +647,19 @@ class IndexWriter(lucene.IndexWriter):
         self.deleteDocuments(parse(*query, **options))
     def __iadd__(self, directory):
         "Add directory (or reader, searcher, writer) to index."
-        if isinstance(directory, basestring):
-            with contextlib.closing(lucene.FSDirectory.open(lucene.File(directory))) as directory:
-                self.addIndexesNoOptimize([directory])
-        else:
-            self.addIndexesNoOptimize([getattr(directory, 'directory', directory)])
+        ref = closing()
+        directory = ref.directory(directory)
+        self.addIndexesNoOptimize([directory if isinstance(directory, lucene.Directory) else directory.directory])
         return self
+    @contextlib.contextmanager
+    def snapshot(self, id=''):
+        "Return context manager of an index commit snapshot."
+        args = [id][:hasattr(lucene, 'IndexWriterConfig')]
+        commit = self.policy.snapshot(*args)
+        try:
+            yield commit
+        finally:
+            self.policy.release(*args)
 
 class Indexer(IndexWriter):
     """An all-purpose interface to an index.
@@ -678,9 +690,6 @@ class Indexer(IndexWriter):
         return iter(self.indexSearcher)
     def __getitem__(self, id):
         return self.indexSearcher[id]
-    @property
-    def segments(self):
-        return self.indexSearcher.segments
     def refresh(self, **caches):
         "Store refreshed searcher with :meth:`IndexSearcher.reopen` caches."
         self.indexSearcher = self.indexSearcher.reopen(**caches)
