@@ -49,14 +49,17 @@ import collections
 import itertools, operator
 import os, optparse
 from email.utils import formatdate
-from contextlib import contextmanager
+import contextlib
 try:
     import simplejson as json
 except ImportError:
     import json
 import lucene
 import cherrypy
-import engine
+try:
+    from . import engine
+except ValueError:
+    import engine
 
 def tool(hook):
     "Return decorator to register tool at given hook point."
@@ -201,7 +204,7 @@ class AttachedMonitor(cherrypy.process.plugins.Monitor):
             callback()
         cherrypy.process.plugins.Monitor.__init__(self, bus, run, frequency)
 
-@contextmanager
+@contextlib.contextmanager
 def HTTPError(status, *exceptions):
     "Interpret exceptions as an HTTPError with given status code."
     try:
@@ -222,10 +225,6 @@ class WebSearcher(object):
         self = object.__new__(cls)
         self.args, self.kwargs = args, kwargs
         return self
-    def init(self, vmargs='-Xrs', **kwargs):
-        "Callback to initialize VM and root object after daemonizing."
-        lucene.initVM(vmargs=vmargs, **kwargs)
-        self.__init__(*self.__dict__.pop('args'), **self.__dict__.pop('kwargs'))
     def close(self):
         self.searcher.close()
     @cherrypy.expose
@@ -262,7 +261,7 @@ class WebSearcher(object):
         """Return ids or documents.
         
         **GET** /docs
-            Return list of doc ids.
+            Return array of doc ids.
             
             :return: [*int*,... ]
         
@@ -272,7 +271,7 @@ class WebSearcher(object):
             
             &fields=\ *chars*,... &fields.multi=\ *chars*,... &fields.indexed=\ *chars*\ [:*chars*],...
             
-            :return: {*string*: *string*\|\ *array*,... }
+            :return: {*string*: *string*\|\ *number*\|\ *array*,... }
         """
         searcher = self.searcher
         if not path:
@@ -341,7 +340,7 @@ class WebSearcher(object):
                 | "query": *string*,
                 | "count": *int*\|null,
                 | "maxscore": *number*\|null,
-                | "docs": [{"__id__": *int*, "__score__": *number*, "__highlights__": {*string*: *array*,... }, *string*: *string*\|\ *array*,... },... ],
+                | "docs": [{"__id__": *int*, "__score__": *number*, "__highlights__": {*string*: *array*,... }, *string*: *object*,... },... ],
                 | "facets": {*string*: {*string*: *int*,... },... },
                 | "groups": [{"count": *int*, "value": *value*, "docs": [*object*,... ]},... ]
                 | "spellcheck": {*string*: {*string*: [*string*,... ],... },... },
@@ -389,34 +388,23 @@ class WebSearcher(object):
             fields = {}
         else:
             hits.fields = lucene.MapFieldSelector(list(itertools.chain(fields, multi)))
-        docs = []
-        groups = collections.defaultdict(lambda: {'docs': [], 'count': 0, 'index': len(groups)})
-        gcount = options.get('group.count', 1)
-        glimit = options.get('group.limit', float('inf'))
-        if group:
-            with HTTPError(httplib.BAD_REQUEST, AttributeError):
-                group = searcher.comparator(*group.split(':'))
-            ids, scores = [], []
-            for id, score in hits.items():
-                item = groups[group[id]]
-                item['count'] += 1
-                if item['count'] <= gcount and item['index'] < glimit:
-                    ids.append(id)
-                    scores.append(score)
-            hits.ids, hits.scores = ids, scores
-        for hit in hits:
-            doc = hit.dict(*multi, **fields)
-            doc.update((name, indexed[name][hit.id]) for name in indexed)
-            fragments = (hl[name].fragments(hit.id, hlcount) for name in hl)
-            if hl:
-                doc['__highlights__'] = dict((name, value) for name, value in zip(hl, fragments) if value is not None)
-            (groups[group[hit.id]]['docs'] if group else docs).append(doc)
-        for name in groups:
-            groups[name]['value'] = name
-        if group:
-            result['groups'] = sorted(groups.values(), key=lambda item: item.pop('index'))
-        else:
-            result['docs'] = docs
+        with HTTPError(httplib.BAD_REQUEST, AttributeError):
+            groups = hits.groupby(searcher.comparator(*group.split(':')).__getitem__) if group else [hits]
+        result['groups'], limit = [], options.get('group.limit', len(groups))
+        for hits in groups[:limit]:
+            docs = []
+            for hit in hits[:options.get('group.count', 1) if group else None]:
+                doc = hit.dict(*multi, **fields)
+                doc.update((name, indexed[name][hit.id]) for name in indexed)
+                fragments = (hl[name].fragments(hit.id, hlcount) for name in hl)
+                if hl:
+                    doc['__highlights__'] = dict((name, value) for name, value in zip(hl, fragments) if value is not None)
+                docs.append(doc)
+            result['groups'].append({'docs': docs, 'count': len(hits), 'value': getattr(hits, 'value', None)})
+        for hits in groups[limit:]:
+            result['groups'].append({'docs': [], 'count': len(hits), 'value': hits.value})
+        if not group:
+            result['docs'] = result.pop('groups')[0]['docs']
         q = q or lucene.MatchAllDocsQuery()
         if facets:
             facets = (tuple(facet.split(':')) if ':' in facet else facet for facet in facets)
@@ -549,7 +537,9 @@ class WebIndexer(WebSearcher):
             self.refresh()
         return {unicode(self.indexer.directory): len(self.indexer)}
     @cherrypy.expose
-    def update(self, **options):
+    @cherrypy.tools.json(process_body=lambda body: dict.fromkeys(body, True))
+    @cherrypy.tools.allow(methods=['POST', 'PUT', 'DELETE'])
+    def update(self, *path, **options):
         """Commit index changes and refresh index version.
         
         **POST** /update
@@ -558,11 +548,23 @@ class WebIndexer(WebSearcher):
             ["expunge"|"optimize",... ]
             
             :return: *int*
+        
+        **PUT, DELETE** /update/snapshot
+            Snapshot current index commit and return array of referenced filenames, or release previous snapshot.
+            
+            :return: [*string*,... ]
         """
-        self.indexer.commit(**options)
-        self.updated = time.time()
-        return len(self.indexer)
-    update._cp_config = dict(WebSearcher.update._cp_config)
+        allow(('PUT', 'DELETE') if path else ('GET', 'POST')) # allow direct method call
+        if not path:
+            self.indexer.commit(**options)
+            self.updated = time.time()
+            return len(self.indexer)
+        path = path[:hasattr(lucene, 'IndexWriterConfig')]
+        with HTTPError(httplib.CONFLICT, lucene.JavaError):
+            if cherrypy.request.method == 'PUT':
+                cherrypy.response.status = httplib.CREATED
+                return list(self.indexer.policy.snapshot(*path).fileNames)
+            self.indexer.policy.release(*path)
     @cherrypy.expose
     @cherrypy.tools.allow(methods=['GET', 'HEAD', 'POST', 'PUT', 'DELETE'])
     def docs(self, *path, **options):
@@ -571,12 +573,12 @@ class WebIndexer(WebSearcher):
         **POST** /docs
             Add documents to index.
             
-            [{*string*: *string*\|\ *array*,... },... ]
+            [{*string*: *string*\|\ *number*\|\ *array*,... },... ]
         
         **PUT, DELETE** /docs/*chars*/*chars*
             Set or delete document.  Unique term should be indexed and is added to the new document.
             
-            {*string*: *string*\|\ *array*,... }
+            {*string*: *string*\|\ *number*\|\ *array*,... }
         """
         with HTTPError(httplib.NOT_FOUND, IndexError):
             allow([('GET', 'HEAD', 'POST'), ('GET', 'HEAD'), ('GET', 'HEAD', 'PUT', 'DELETE')][len(path)])
@@ -645,6 +647,25 @@ class WebIndexer(WebSearcher):
             field = self.indexer.fields[name]
         return dict((name, str(getattr(field, name))) for name in ['store', 'index', 'termvector'])
 
+def init(vmargs='-Xrs', **kwargs):
+    "Callback to initialize VM and app roots after daemonizing."
+    lucene.initVM(vmargs=vmargs, **kwargs)
+    for app in cherrypy.tree.apps.values():
+        if isinstance(app.root, WebSearcher):
+            app.root.__init__(*app.root.__dict__.pop('args'), **app.root.__dict__.pop('kwargs'))
+
+def mount(root, path='', config=None, autoupdate=0):
+    """Attach root and subscribe to plugins.
+    
+    :param root,path,config: see cherrypy.tree.mount
+    :param autoupdate: see command-line options
+    """
+    if hasattr(root, 'close'):
+        cherrypy.engine.subscribe('stop', root.close)
+    if autoupdate:
+        AttachedMonitor(cherrypy.engine, root.update, autoupdate).subscribe()
+    return cherrypy.tree.mount(root, path, config)
+
 def start(root=None, path='', config=None, pidfile='', daemonize=False, autoreload=0, autoupdate=0, callback=None):
     """Attach root, subscribe to plugins, and start server.
     
@@ -653,8 +674,6 @@ def start(root=None, path='', config=None, pidfile='', daemonize=False, autorelo
     :param callback: optional callback function scheduled after daemonizing
     """
     cherrypy.engine.subscribe('start_thread', attach_thread)
-    if hasattr(root, 'close'):
-        cherrypy.engine.subscribe('stop', root.close)
     cherrypy.config['engine.autoreload.on'] = False
     if pidfile:
         cherrypy.process.plugins.PIDFile(cherrypy.engine, os.path.abspath(pidfile)).subscribe()
@@ -663,14 +682,14 @@ def start(root=None, path='', config=None, pidfile='', daemonize=False, autorelo
         cherrypy.process.plugins.Daemonizer(cherrypy.engine).subscribe()
     if autoreload:
         Autoreloader(cherrypy.engine, autoreload).subscribe()
-    if autoupdate:
-        AttachedMonitor(cherrypy.engine, root.update, autoupdate).subscribe()
     if callback:
         priority = (cherrypy.process.plugins.Daemonizer.start.priority + cherrypy.server.start.priority) // 2
         cherrypy.engine.subscribe('start', callback, priority)
-    cherrypy.quickstart(root, path, config)
+    if root is not None:
+        mount(root, path, config, autoupdate)
+    cherrypy.quickstart(cherrypy.tree.apps.get(path), path, config)
 
-parser = optparse.OptionParser(usage='python %prog [index_directory ...]')
+parser = optparse.OptionParser(usage='python -m lupyne.server [index_directory ...]')
 parser.add_option('-r', '--read-only', action='store_true', help='expose only read methods; no write lock')
 parser.add_option('-c', '--config', help='optional configuration file or json object of global params')
 parser.add_option('-p', '--pidfile', metavar='FILE', help='store the process id in the given file')
@@ -685,4 +704,4 @@ if __name__ == '__main__':
         options.config = {'global': json.loads(options.config)}
     cls = WebSearcher if (read_only or len(args) > 1) else WebIndexer
     root = cls.new(*map(os.path.abspath, args))
-    start(root, callback=root.init, **options.__dict__)
+    start(root, callback=init, **options.__dict__)
