@@ -3,12 +3,9 @@ import unittest
 import os, sys
 import lucene
 
-if __package__ == '':   # support testing in-place
-    sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
-
-from local import TestCase as local
-from remote import TestCase as remote
-from distributed import TestCase as distributed
+from .local import TestCase as local
+from .remote import TestCase as remote
+from .distributed import TestCase as distributed
 
 class TestRunner(unittest.TextTestRunner):
     def run(self, test):
