@@ -7,10 +7,11 @@ import collections
 import warnings
 import datetime
 import math
-from contextlib import contextmanager
+import bisect
+import contextlib
 import lucene
 from lupyne import engine
-import fixture
+from . import fixture
 
 class typeAsPayload(engine.TokenFilter):
     "Custom implementation of lucene TypeAsPayloadTokenFilter."
@@ -19,7 +20,7 @@ class typeAsPayload(engine.TokenFilter):
         self.payload = self.type.encode('utf8')
         return result
 
-@contextmanager
+@contextlib.contextmanager
 def assertWarns(*categories):
     with warnings.catch_warnings(record=True) as messages:
         yield
@@ -29,9 +30,8 @@ def assertWarns(*categories):
 
 class Filter(lucene.PythonFilter):
     "Broken filter to test errors are raised."
-    def getBitSet(self, indexReader):
+    def getDocIdSet(self, indexReader):
         assert False
-    getDocIdSet = getBitSet
 
 class BaseTest(unittest.TestCase):
     def setUp(self):
@@ -71,9 +71,6 @@ class TestCase(BaseTest):
         assert not indexer.optimized
         indexer.commit()
         assert searcher is not indexer.indexSearcher
-        with assertWarns(DeprecationWarning, DeprecationWarning):
-            searcher = engine.ParallelMultiSearcher([indexer.indexSearcher, indexer.directory])
-        assert searcher.count() == 2 * len(indexer)
         assert list(indexer) == [0]
         assert indexer.current and indexer.optimized
         assert 0 in indexer and 1 not in indexer
@@ -146,8 +143,6 @@ class TestCase(BaseTest):
         assert 0 in indexer and len(indexer) == 1 and indexer.segments == {'_0': 1}
         indexer.commit()
         assert 0 not in indexer and len(indexer) == 0 and sum(indexer.segments.values()) == 0
-        with assertWarns(DeprecationWarning):
-            assert isinstance(super(engine.Indexer, indexer).segments, dict)
         indexer.add(tag='test', name='old')
         indexer.update('tag', boost=2.0, tag='test')
         indexer.commit()
@@ -205,9 +200,12 @@ class TestCase(BaseTest):
         doc['date'] = engine.Analyzer(lucene.WhitespaceTokenizer).tokens(doc['date']), 2.0
         scores = list(searcher.match(doc, 'text:congress', 'text:law', 'amendment:27', 'date:19*'))
         assert 0.0 == scores[0] < scores[1] < scores[2] < scores[3] == 1.0
-        searcher = engine.MultiSearcher([indexer.directory, self.tempdir])
+        searcher = engine.MultiSearcher([indexer.indexReader, self.tempdir])
         assert searcher.count() == len(searcher) == 2 * len(indexer)
-        searcher = searcher.reopen()
+        searcher.sorters['amendment'] = engine.SortField('amenmdment', int)
+        comparator = searcher.comparator('amendment')
+        assert comparator is searcher.comparator('amendment') and set(map(type, comparator)) == set([int])
+        assert searcher is searcher.reopen()
         assert searcher.facets(lucene.MatchAllDocsQuery(), 'amendment')['amendment'] == dict.fromkeys(map(str, range(1, 28)), 2)
         reader = searcher.indexReader
         del searcher
@@ -269,7 +267,7 @@ class TestCase(BaseTest):
             indexer.filters[name] = engine.Query.prefix(name, '').filter()
         query = engine.Query.term('text', 'right')
         assert indexer.facets(str(query), 'amendment', 'article') == {'amendment': 12, 'article': 1}
-        self.assertRaises(lucene.InvalidArgsError, indexer.overlap, query.filter(), query.filter(cache=False))
+        self.assertRaises(TypeError, indexer.overlap, query.filter(), query.filter(cache=False))
         hits = indexer.search('text:people', filter=query.filter())
         assert len(hits) == 4
         hit, = indexer.search('date:192*')
@@ -303,6 +301,8 @@ class TestCase(BaseTest):
         assert len(query) == len(list(query)) == 2
         span = engine.Query.span('text', 'persons')
         count = indexer.count(span)
+        if hasattr(lucene, 'SpanMultiTermQueryWrapper'):
+            assert indexer.count(engine.Query.span(engine.Query.prefix('text', 'person'))) > count
         near = engine.Query.near('text', 'persons', 'papers', slop=1, inOrder=False)
         assert indexer.count(span - near) == count
         near = span.near(engine.Query.span('text', 'papers') | engine.Query.span('text', 'things'), slop=1)
@@ -346,8 +346,22 @@ class TestCase(BaseTest):
         query = indexer.parse('"hello world"', field='text', spellcheck=True)
         assert lucene.PhraseQuery.instance_(query) and str(query) == 'text:"held would"'
         assert str(indexer.parse('vwxyz', field='text', spellcheck=True)) == 'text:vwxyz'
+        with indexer.snapshot() as commit:
+            self.assertRaises(lucene.JavaError, indexer.snapshot().__enter__)
+        files = set(os.listdir(self.tempdir))
+        path = os.path.join(self.tempdir, 'temp')
+        with indexer.snapshot('backup') as commit:
+            indexer.commit(optimize=True)
+            assert indexer.indexCommit.generation > commit.generation
+            engine.indexers.copy(commit, path)
+            assert set(os.listdir(path)) == set(commit.fileNames) < files < set(os.listdir(self.tempdir))
+            filepath = os.path.join(path, commit.segmentsFileName)
+            os.remove(filepath)
+            open(filepath, 'w').close()
+            self.assertRaises(OSError, engine.indexers.copy, commit, path)
         del indexer
         assert engine.Indexer(self.tempdir)
+        assert not os.path.exists(os.path.join(self.tempdir, commit.segmentsFileName))
     
     def testAdvanced(self):
         "Large data set with hierarchical fields."
@@ -407,14 +421,16 @@ class TestCase(BaseTest):
         indexer = engine.Indexer(self.tempdir, 'w')
         for name, params in fixture.zipcodes.fields.items():
             indexer.set(name, **params)
+        for name in ('longitude', 'latitude'):
+            indexer.set(name, engine.NumericField, store=True)
         field = indexer.fields['tile'] = engine.PointField('tile', precision=15, step=2, store=True)
         points = []
         for doc in fixture.zipcodes.docs():
             if doc['state'] == 'CA':
-                lat, lng = doc.pop('latitude'), doc.pop('longitude')
-                indexer.add(doc, tile=[(lng, lat)], latitude=str(lat), longitude=str(lng))
+                point = doc['longitude'], doc['latitude']
+                indexer.add(doc, tile=[point])
                 if doc['city'] == 'Los Angeles':
-                    points.append((lng, lat))
+                    points.append(point)
         assert len(list(engine.PolygonField('', precision=15).items(points))) > len(points)
         indexer.commit()
         city, zipcode, tile = 'Beverly Hills', '90210', '023012311120332'
@@ -434,13 +450,26 @@ class TestCase(BaseTest):
         assert city in cities and len(cities) > 10
         query = field.within(x, y, 10**4)
         assert len(query) < 3
-        cities = set(hit['city'] for hit in indexer.search(query))
+        distances = indexer.distances(x, y, 'longitude', 'latitude')
+        hits = indexer.search(query, sort=distances.__getitem__)
+        assert hits[0]['zipcode'] == zipcode and distances[hits[0].id] < 10
+        cities = set(hit['city'] for hit in hits)
         assert city in cities and 100 > len(cities) > 50
-        hits = indexer.search(field.within(x, y, 10**5))
+        hits = indexer.search(field.within(x, y, 10**5, limit=100))
         cities = set(hit['city'] for hit in hits)
         assert city in cities and len(cities) > 100
+        ranges = 10**2, 10**5
+        groups = hits.groupby(lambda id: bisect.bisect_left(ranges, distances[id]))
+        counts = dict((hits.value, len(hits)) for hits in groups)
+        assert 1 == counts[0] < counts[2] < counts[1]
         assert len(field.within(x, y, 10**8)) == 1
-        del indexer
+        if hasattr(lucene, 'LatLongDistanceFilter'):
+            f = field.filter(x, y, 10**4, 'longitude', 'latitude')
+            ids = indexer.search(query, sort=distances.__getitem__).ids
+            hits = indexer.search(count=10, filter=f, sort=f.sorter())
+            assert len(hits) < len(ids) and hits.ids == ids[:len(hits)]
+            assert all(f[id] <= distances[id] < 10**4 for id in hits.ids)
+            self.assertRaises(KeyError, f.__getitem__, -1)
     
     def testFields(self):
         "Custom fields."
@@ -485,9 +514,9 @@ class TestCase(BaseTest):
         assert indexer.count(query) == len(sizes) - len(ids)
         indexer.sorters['year'] = engine.SortField('Y-m-d', type=int, parser=lambda date: int(date.split('-')[0]))
         assert indexer.comparator('year')[:10] == [1791] * 10
+        cache = len(lucene.FieldCache.DEFAULT.cacheEntries)
         hits = indexer.search(count=3, sort='year')
         assert [int(hit['amendment']) for hit in hits] == [1, 2, 3]
-        cache = len(lucene.FieldCache.DEFAULT.cacheEntries)
         hits = indexer.search(count=3, sort='year', reverse=True)
         assert [int(hit['amendment']) for hit in hits] == [27, 26, 25]
         assert cache == len(lucene.FieldCache.DEFAULT.cacheEntries)
@@ -545,6 +574,8 @@ class TestCase(BaseTest):
             assert len(sizes) == count and all(isinstance(size, int) for size in sizes)
             numbers = dict(indexer.numbers('size', step, type=float, counts=True))
             assert sum(numbers.values()) == len(indexer) and all(isinstance(number, float) for number in numbers)
+        hit, = indexer.search(indexer.fields['amendment'].term(1))
+        assert hit['amendment'] == '1'
     
     def testHighlighting(self):
         "Highlighting text fragments."
