@@ -13,7 +13,7 @@ from email.utils import parsedate
 import lucene
 import cherrypy
 from lupyne import client, engine, server
-import fixture, local
+from . import fixture, local
 
 @contextlib.contextmanager
 def assertRaises(exception, code):
@@ -87,6 +87,8 @@ class TestCase(BaseTest):
             resource.post('/terms')
         with assertRaises(httplib.HTTPException, httplib.METHOD_NOT_ALLOWED):
             resource.get('/update')
+        with assertRaises(httplib.HTTPException, httplib.METHOD_NOT_ALLOWED):
+            resource.post('/update/snapshot')
         with assertRaises(httplib.HTTPException, httplib.METHOD_NOT_ALLOWED):
             resource.put('/fields')
         with assertRaises(httplib.HTTPException, httplib.METHOD_NOT_ALLOWED):
@@ -248,14 +250,23 @@ class TestCase(BaseTest):
         assert resource.post('/', [self.tempdir]).values() == [2]
         with local.assertWarns(DeprecationWarning, UserWarning):
             assert Resource(resource.host, resource.port).call('GET', '/missing', redirect=True)()
+        response = resource.call('PUT', '/update/snapshot')
+        assert response.status == httplib.CREATED
+        assert all(name.startswith('_') or name.startswith('segments_') for name in response())
+        with assertRaises(httplib.HTTPException, httplib.CONFLICT):
+            resource.put('/update/snapshot')
+        assert not resource.delete('/update/snapshot')
+        with assertRaises(httplib.HTTPException, httplib.CONFLICT):
+            resource.delete('/update/snapshot')
         resource = client.Resource('localhost', self.ports[-1] + 1)
         with assertRaises(socket.error, errno.ECONNREFUSED):
             resource.get('/')
         self.stop(self.servers.pop(0))
         pidfile = os.path.join(self.tempdir, 'pid')
         server = self.start(self.ports[0], '-dp', pidfile)
-        assert server.poll() == 0
         os.kill(int(open(pidfile).read()), signal.SIGKILL)
+        filepath = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'lupyne/server.py')
+        assert subprocess.call((sys.executable, filepath, '-x'), stderr=subprocess.PIPE)
     
     def testBasic(self):
         "Remote text indexing and searching."
@@ -451,6 +462,7 @@ class TestCase(BaseTest):
         response = resource.call('GET', '/docs')
         assert response and version != response.getheader('etag')
         cherrypy.engine.exit()
+        assert root.update() == 1
 
 if __name__ == '__main__':
     lucene.initVM()
